import React, {
  FC,
  createContext,
  useEffect,
  useState,
  useContext,
  ReactNode,
} from 'react';
import * as RNLocalize from 'react-native-localize';
import i18n from 'i18n-js';
import memoize from 'lodash.memoize';

export type LanguageContextProviderProps = {
  children: ReactNode;
  translationGetters: { [key: string]: any };
};
export type langTypes = 'en' | 'zh-cmn-Hant-HK';
export type LanguageContextProps = {
  selected: langTypes;
  setSelected: (val: langTypes) => void;
  translate: (val: string) => string;
  translateText: (val: string) => string;
};

export const LanguageContext = createContext<{ current: LanguageContextProps }>(
  {}
);
export const LanguageContextProvider: FC<LanguageContextProviderProps> = ({
  children,
  translationGetters,
}: LanguageContextProviderProps) => {
  const [selected, setSelected] = useState<langTypes>('en');

  const translate = memoize(
    (key, config) => i18n.t(key, config),
    (key, config) => (config ? key + JSON.stringify(config) : key)
  );
  const setI18nConfig = () => {
    const fallback = { languageTag: selected as string };
    console.log('inside iiiii1888 ', selected);
    const { languageTag } =
      // RNLocalize.findBestAvailableLanguage(Object.keys(translationGetters)) ||
      fallback;

    translate.cache.clear();

    i18n.translations = { [languageTag]: translationGetters[languageTag]() };
    i18n.locale = languageTag;
  };
  setI18nConfig();
  const handleLocalizationChange = () => {
    console.log('inside wickdd ', selected);

    setI18nConfig();
  };
  useEffect(() => {
    RNLocalize.addEventListener('change', handleLocalizationChange);
    return RNLocalize.removeEventListener('change', handleLocalizationChange);
  }, [selected]);
  const translateText = (val: string): string => {
    if (val === '' || val === null) {
      return '';
    } else {
      if (selected === 'en') {
        return val;
      }
      console.log(
        'test translation errors ',
        translate(val),
        translate(val).startsWith(`[missing "${selected}.`)
      );
      if (translate(val).startsWith(`[missing "${selected}.`)) {
        return val;
      }
      return translate(val);
    }
  };
  return (
    <LanguageContext.Provider
      value={{ current: { selected, setSelected, translate, translateText } }}
    >
      {children}
    </LanguageContext.Provider>
  );
};

const useLanguage = () => {
  const context = useContext<{ current?: LanguageContextProps }>(
    LanguageContext
  );
  return context?.current!;
};
export default useLanguage;
